% ------------------------------------------------------------------------------%
% This is code for a computational experiment that compares the performance     %
% of a regular Particle Filter(PF), a Gaussian Particle Filter(GPF)             %
% and a Gaussian Particle Filter that samples directly from the posterior(GPF2) %
% ------------------------------------------------------------------------------%
% For more details on GPF and GPF2, see                                         %
% [Frank Hutter and Richard Dearden, Efficient On-Line Fault Diagnosis for      %
% Non-Linear Systems in "Proceedings of The 11th International Conference on    %
% AI, Robotics and Automation in Space(i-SAIRAS03)"].                           %
% ------------------------------------------------------------------------------%
% Last update : January-08, 2003  ( draft version )                             %
% AUTHORS: Frank Hutter, Richard Dearden                                        %
% ------------------------------------------------------------------------------%
clear all;
echo off;
path('./core',path);
path('./algos',path);
path('./general',path);

% =======================================================================
%     CHOOSE THE MODEL YOU WANT TO WORK WITH.
% Make sure not to have more than one of those models in the path !! (restart Matlab)
% =======================================================================

% Model applied for our GPF paper.
path('./model_for_gpf_paper',path);

% For this model of real data, only the plot of the discrete modes makes sense. 
% For the other plots, there's no ground truth to compare with, so they're pointless !
%path('./model_for_real_data',path);         

% Linear model we applied for Nando's RBPF paper.
%path('./linear_model_for_nandos_paper',path);

% =======================================================================
%     INITIALISATION
% =======================================================================
par = initParameters;

% =============================================================================
%     READ DATA FROM FILE
% =============================================================================
[T,u,x,y,z] = readData(par);
S=3;                       % First data point that is to be plotted.
%T = 270-1+S;               % Last entry we use from the data set.

% =============================================================================
%     THE EXPERIMENT
% =============================================================================
N_min = 1;                 % Minimal number of particles. 
N_max = 128;               % Maximal number of particles.
runs =  25;                % With each number of particles we do that many runs.

% pf = importdata('pf-results.dat');
% upf = importdata('upf-results.dat');
% gpf = importdata('gpf-results.dat');
% gpf2 = importdata('gpf2-results.dat');    

pf(1) = measurePerformance('pfalgo',par,u,x,y,z,S,T,1,runs);
upf(1) = measurePerformance('upfalgo',par,u,x,y,z,S,T,1,runs);
gpf(1) = measurePerformance('gpfalgo',par,u,x,y,z,S,T,1,runs);
gpf2(1) = measurePerformance('gpf2algo',par,u,x,y,z,S,T,1,runs);

for logN = log2(N_min):log2(N_max)
    N = 2^logN;
    disp(' ');
    disp(['Estimation has started for ', num2str(N), ' particle(s)']);
    disp('PF:');
    pf(logN+1) = measurePerformance('pfalgo',par,u,x,y,z,S,T,N,runs);
    disp('UPF:');
    if N > 16 % Only do UPF up to 16 particles.
        upf(logN+1) = upf(logN);
    else
        upf(logN+1) = measurePerformance('upfalgo',par,u,x,y,z,S,T,N,runs);
    end
    disp('GPF:');
    if N > 16 % Only do GPF up to 16 particles. 
        gpf(logN+1) = gpf(logN);
    else
        gpf(logN+1) = measurePerformance('gpfalgo',par,u,x,y,z,S,T,N,runs);
    end
    disp('GPF2:');
    if N > 4 % Only do GPF2 up to 4 particles.
        gpf2(logN+1) = gpf2(logN);
    else
        gpf2(logN+1) = measurePerformance('gpf2algo',par,u,x,y,z,S,T,N,runs);
    end
    disp(['Errors for ', num2str(N), ' particles: (', ...
            num2str(pf(logN+1).mean_error), ', ', ...
            num2str(upf(logN+1).mean_error), ', ', ...
            num2str(gpf(logN+1).mean_error), ', ', ...
            num2str(gpf2(logN+1).mean_error), ')']);

    disp(['Mean STDDEVs between ', num2str(N), ' particles: (', ...
        num2str(pf(logN+1).stddev_errors_between_particles), ', ', ...
        num2str(upf(logN+1).stddev_errors_between_particles), ', ', ...
        num2str(gpf(logN+1).stddev_errors_between_particles), ', ', ...
        num2str(gpf2(logN+1).stddev_errors_between_particles), ')']);
    
    disp(['STDDEVs between runs for ', num2str(runs), ' runs: (', ...
            num2str(pf(logN+1).stddev_errors_between_runs), ', ', ...
            num2str(upf(logN+1).stddev_errors_between_runs), ', ', ...
            num2str(gpf(logN+1).stddev_errors_between_runs), ', ', ...
            num2str(gpf2(logN+1).stddev_errors_between_runs), ')']);
    
    disp(['MSEs for ', num2str(N), ' particles: (', ...
            num2str(pf(logN+1).mean_mse), ', ', ...
            num2str(upf(logN+1).mean_mse), ', ', ...
            num2str(gpf(logN+1).mean_mse), ', ', ...
            num2str(gpf2(logN+1).mean_mse), ')']);
 
    disp(['Times per time step for ', num2str(N), ' particles: (', ...
            num2str(pf(logN+1).mean_time), ', ', ...
            num2str(upf(logN+1).mean_time), ', ', ...
            num2str(gpf(logN+1).mean_time), ', ', ...
            num2str(gpf2(logN+1).mean_time), ')']);

end
plotNiceFigures;